// お習字ロボットアーム

package org.microbridge.servocontrol;

import java.util.ArrayList;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;

// お習字画面のビュー
public class BrushView extends View implements OnTouchListener
{
	// お習字ロボットアームアプリ本体
	private RobotArm parent;

	private Paint paint;
	private ArrayList<Path> pathList = new ArrayList<Path>();
	private Path path;

	public BrushView(Context context, AttributeSet attrs){
		super(context, attrs);
		initial();
	}

	public BrushView(Context context)
	{
		super(context);
		initial();
	}
	
	// 初期化
	private void initial()
	{
		paint = new Paint();
		paint.setColor(Color.BLACK);
		paint.setStyle(Paint.Style.STROKE);
		paint.setAntiAlias(true);
		paint.setStrokeWidth(10);

		setFocusable(true);
		setFocusableInTouchMode(true);

		this.setOnTouchListener(this);
	}
	
	// アプリ本体をセット
	public void setParent(RobotArm robotArm) {
		this.parent = robotArm;
	}
	
	// 描画のとき
	@Override
	public void onDraw(Canvas canvas)
	{
		super.onDraw(canvas);
		// 一画ずつ、運筆を描画
		for (Path path : pathList) {
			canvas.drawPath(path, paint);
		}
	}
	
	// タッチのとき
//	@Override
	public boolean onTouch(View v, MotionEvent event)
	{
		// 記録中でないなら何もしない
		if(!parent.isRecording) return true;
		
		// タッチの座標
		float sx = event.getX();
		float sy = event.getY();

		switch (event.getAction()) {
		// 指が触れたとき
		case MotionEvent.ACTION_DOWN:
			// 新しい一画
			path = new Path();
			path.moveTo(sx, sy);
			pathList.add(path);
			// アプリ本体に通知
			parent.touchDown(sx, sy);
			break;
		// 指が動いたとき
		case MotionEvent.ACTION_MOVE:
			// 線を描く
			path.lineTo(sx, sy);
			// アプリ本体に通知
			parent.touchMove(sx, sy);
			break;
		// 指が離れたとき
		case MotionEvent.ACTION_UP:
			// 線を描く
			path.lineTo(sx, sy);
			// アプリ本体に通知
			parent.touchUp(sx, sy);
			break;
		default:
		}
		invalidate(); // 再描画
		return true;
	}

	// 運筆データをクリアして画面を再描画
	public void clearPath()
	{
		pathList.clear();
		invalidate();
	}
}
