// お習字ロボットアーム

package org.microbridge.servocontrol;

// 逆キネマティックス
public class InvKinema {
	// 画面領域(液晶)の座標[pixel]
	public float sx;
	public float sy;
	// 画面領域(液晶)のサイズ[pixel]
	public static float sWidth;
	public static float sHeight;
	// 物理領域(紙)のサイズ[mm]
	public static float width;
	public static float height;
	// 物理領域(紙)のオフセット[mm]
	public static float X0;
	public static float Y0;
	// 二の腕の長さ[mm]
	public static float L;
	// 手首から先の長さ[mm]
	public static float d;
	// 各サーボの角度 [deg] (0-180)
	public float theta;	// 肩ヨー軸
	public float phi1;	// 肩ピッチ軸
	public float phi2;	// 肘ピッチ軸
	public float phi3;	// 手首ピッチ軸

	InvKinema(int sx, int sy)
	{
		this.sx = sx;
		this.sy = sy;
	}

	InvKinema(float theta, float phi1, float phi2, float phi3)
	{
		this.theta = theta;
		this.phi1 = phi1;
		this.phi2 = phi2;
		this.phi3 = phi3;
	}

	// 逆キネの計算（座標から各サーボの角度を求める）
	void calcAxes() throws Exception
	{
		// 画面領域(液晶)の座標から物理領域(紙)の座標に換算
		float x;
		float y;
		x = X0 + (sx * width)/sWidth;
		y = Y0 - (sy * height)/sHeight;
		if(y<0) throw new Exception("y must be >=0. y=" + y);

		// ヨー軸(真上から見た角度)の計算
		double th;
		if(x > 0){
			th = Math.atan(y/x);
		}else if(x<0){
			th = Math.atan(y/x) + Math.PI;
		}else{
			th = Math.PI / 2.0;
		}
		theta = (float)((th * 180.0) / Math.PI);

		// ピッチ軸(真横から見た角度)の計算
		double phi;
		phi = Math.acos( (Math.sqrt(x*x + y*y) - d) / (2*L) );
		phi1 = (float)((phi * 180.0) / Math.PI);
		phi2 = (float)(((Math.PI - 2*phi) * 180.0) / Math.PI);
		phi3 = (float)((phi * 180.0) / Math.PI);

		if((phi1<0) || (phi1>180)) throw new Exception("phi1 must be 0-180. phi1=" + phi1);
		if((phi2<0) || (phi2>180)) throw new Exception("phi2 must be 0-180. phi2=" + phi2);
		if((phi3<0) || (phi3>180)) throw new Exception("phi3 must be 0-180. phi3=" + phi3);
	}
	
	// 筆を持ち上げたときのサーボの角度計算 (肩を10°上げる)
	InvKinema bringUp()
	{
		float th = this.theta;
		float p1 = this.phi1;
		float p2 = this.phi2;
		float p3 = this.phi3;
		p1 = p1 + 10;
		if(p1>180) p1=180;
		
		return new InvKinema(th,p1,p2,p3);
	}
}
